#include "http/res.h"
#include "lander.h"
#include "log.h"

// TODO entry leaks if key is already present
static bool add_entry(char **key_ptr, int *key_len_ptr, http_loop_ctx *ctx,
                      Entry *entry, bool random) {
  // The first match group matches the "long" path
  bool secure =
      (ctx->req.regex_groups[1].rm_eo - ctx->req.regex_groups[1].rm_so) == 1;

  char *key;
  int key_len = 0;
  TrieExitCode res;

  if (random) {
    res = trie_add_random(ctx->g->trie, &key, entry, secure);

    if (res == Ok) {
      key_len = strlen(key);
    }
  } else {
    key = (char *)&ctx->req.path[ctx->req.regex_groups[2].rm_so];
    key_len = ctx->req.regex_groups[2].rm_eo - ctx->req.regex_groups[2].rm_so;

    res = trie_add_len(ctx->g->trie, key, key_len, entry);
  }

  switch (res) {
  case Ok:
    break;
  case AlreadyPresent:
    ctx->res.status = http_conflict;
    return false;
  default:
    ctx->res.status = http_internal_server_error;
    return false;
  }

  // Add a slash to the key and add it as the location header
  char *buf = malloc(key_len + 2);

  memcpy(&buf[1], key, key_len);
  buf[0] = '/';
  buf[key_len + 1] = '\0';

  http_res_add_header(&ctx->res, http_header_location, buf, true);

  if (key_ptr != NULL) {
    *key_ptr = key;
  }

  if (key_len_ptr != NULL) {
    *key_len_ptr = key_len;
  }

  return true;
}

bool lander_post_redirect(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;
  bool random =
      ctx->req.regex_groups[2].rm_eo == ctx->req.regex_groups[2].rm_so;

  // Allocate a new buffer to pass to the trie
  char *url = malloc(ctx->req.body.len + 1);
  memcpy(url, ctx->req.body.buf, ctx->req.body.len);
  url[ctx->req.body.len] = '\0';

  Entry *new_entry = entry_new(Redirect, url);

  // The entry duplicates the string
  free(url);

  // We don't check the result here, because we would perform the same action
  // either way
  char *key;
  add_entry(&key, NULL, ctx, new_entry, random);

  if (random) {
    free(key);
  }

  conn->state = event_loop_conn_state_res;

  return true;
}

bool lander_post_paste(event_loop_conn *conn) {
  http_loop_ctx *ctx = conn->ctx;
  bool random =
      ctx->req.regex_groups[2].rm_eo == ctx->req.regex_groups[2].rm_so;

  char *key;
  int key_len;
  Entry *new_entry = entry_new(Paste, "");

  if (!add_entry(&key, &key_len, ctx, new_entry, random)) {
    conn->state = event_loop_conn_state_res;

    return true;
  }

  char *fname = malloc(8 + key_len);
  sprintf(fname, "pastes/%.*s", key_len, key);

  ctx->req.body.fname = fname;
  ctx->req.body.fname_owned = true;

  if (random) {
    free(key);
  }

  return true;
}
